/******************************************************************************

  Copyright(c) 2005 Intel Corporation. All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called LICENSE.

  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/
#include <linux/compiler.h>
#include <linux/config.h>
#include <linux/errno.h>
#include <linux/if_arp.h>
#include <linux/in6.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/proc_fs.h>
#include <linux/skbuff.h>
#include <linux/slab.h>
#include <linux/tcp.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/wireless.h>
#include <linux/etherdevice.h>
#include <asm/uaccess.h>

#include <net/ieee80211.h>


/*
 * NOTE:  The following tables are guidelines meant to improve the user
 * experience by being able to try and verify channels as being valid so that
 * error codes can be more readily provided if channel changes are requested
 * to those not supported by your hardware or geographic location.
 *
 * This code is not meant to be used to enforce regulatory compliance.
 *
 */
static const struct ieee80211_geo ieee80211_geos[IEEE80211_GEO_LAST] = {
	[IEEE80211_GEO_001] = { /* Canada, USA, Mexico, etc. */
		"001",
		.bg_channels = 11,
		.bg = { { 2412, 1}, { 2417, 2}, { 2422, 3},
			{ 2427, 4}, { 2432, 5}, { 2437, 6},
			{ 2442, 7}, { 2447, 8}, { 2452, 9},
			{ 2457,10}, { 2462,11} },
		.a_channels = 8,
		.a = { { 5180, 36}, { 5200, 40},
		       { 5220, 44}, { 5240, 48},
		       { 5260, 52, 1 }, { 5280, 56, 1 },
		       { 5300, 60, 1 }, { 5320, 64, 1 } },
	},
	[IEEE80211_GEO_002] = { /* Europe */
		"002",
		.bg_channels = 13,
		.bg = { { 2412, 1}, { 2417, 2}, { 2422, 3},
			{ 2427, 4}, { 2432, 5}, { 2437, 6},
			{ 2442, 7}, { 2447, 8}, { 2452, 9},
			{ 2457,10}, { 2462,11}, { 2467,12},
			{ 2472,13} },
		.a_channels = 12,
		.a = { { 5180, 36}, { 5200, 40},
		       { 5220, 44}, { 5240, 48},
		       { 5260, 52, 1 }, { 5280, 56, 1 },
		       { 5300, 60, 1 }, { 5320, 64, 1 },
		       { 5745,149}, { 5765,153},
		       { 5785,157}, { 5805,161} },
	},
	[IEEE80211_GEO_003] = { /* Japan -- TO BE FILLED OUT */
		"003",
		.bg_channels = 11,
		.bg = { { 2412, 1}, { 2417, 2}, { 2422, 3},
			{ 2427, 4}, { 2432, 5}, { 2437, 6},
			{ 2442, 7}, { 2447, 8}, { 2452, 9},
			{ 2457,10}, { 2462,11} },
		.a_channels = 0,
	},
	[IEEE80211_GEO_004] = { /* Korea -- TO BE FILLED OUT */
		"004",
		.bg_channels = 11,
		.bg = { { 2412, 1}, { 2417, 2}, { 2422, 3},
			{ 2427, 4}, { 2432, 5}, { 2437, 6},
			{ 2442, 7}, { 2447, 8}, { 2452, 9},
			{ 2457,10}, { 2462,11} },
		.a_channels = 0,
	},
	[IEEE80211_GEO_000] = { /* No geo -- most restricted channels */
		"000",
		.bg_channels = 11,
		.bg = { { 2412, 1}, { 2417, 2}, { 2422, 3},
			{ 2427, 4}, { 2432, 5}, { 2437, 6},
			{ 2442, 7}, { 2447, 8}, { 2452, 9},
			{ 2457,10}, { 2462,11} },
		.a_channels = 4,
		.a = { { 5180, 36}, { 5190, 38},
		       { 5220, 44}, { 5240, 48} },
	}
};

const struct ieee80211_geo *ieee80211_get_geo(struct ieee80211_device *ieee)
{
	if (unlikely(ieee->geography >= IEEE80211_GEO_LAST))
		BUG();

	return &ieee80211_geos[ieee->geography];
}

int ieee80211_is_valid_channel(struct ieee80211_device *ieee, u8 channel)
{
	const struct ieee80211_geo *geo = &ieee80211_geos[ieee->geography];
	int i;

	if (ieee->freq_band & IEEE80211_24GHZ_BAND)
		for (i = 0; i < geo->bg_channels; i++)
			if (geo->bg[i].channel == channel)
				return IEEE80211_24GHZ_BAND;

	if (ieee->freq_band & IEEE80211_52GHZ_BAND)
		for (i = 0; i < geo->a_channels; i++)
			if (geo->a[i].channel == channel)
				return IEEE80211_52GHZ_BAND;

	return 0;
}

int ieee80211_channel_to_index(struct ieee80211_device *ieee, u8 channel)
{
	const struct ieee80211_geo *geo = &ieee80211_geos[ieee->geography];
	int i;

	if (ieee->freq_band & IEEE80211_24GHZ_BAND)
		for (i = 0; i < geo->bg_channels; i++)
			if (geo->bg[i].channel == channel)
				return i;

	if (ieee->freq_band & IEEE80211_52GHZ_BAND)
		for (i = 0; i < geo->a_channels; i++)
			if (geo->a[i].channel == channel)
				return i;

	return -1;
}

u8 ieee80211_freq_to_channel(struct ieee80211_device *ieee, u32 freq)
{
	const struct ieee80211_geo *geo = &ieee80211_geos[ieee->geography];
	int i;

	freq /= 100000;

	if (ieee->freq_band & IEEE80211_24GHZ_BAND)
		for (i = 0; i < geo->bg_channels; i++)
			if (geo->bg[i].freq == freq)
				return geo->bg[i].channel;

	if (ieee->freq_band & IEEE80211_52GHZ_BAND)
		for (i = 0; i < geo->a_channels; i++)
			if (geo->a[i].freq == freq)
				return geo->a[i].channel;

	return 0;
}

EXPORT_SYMBOL(ieee80211_get_geo);
EXPORT_SYMBOL(ieee80211_is_valid_channel);
EXPORT_SYMBOL(ieee80211_freq_to_channel);
EXPORT_SYMBOL(ieee80211_channel_to_index);
